unit Login;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics,
  Controls, Forms, Dialogs, DB, DBTables, StdCtrls, Buttons,
  Global;

type
  TLoginDialogForm = class(TForm)
    UserNameEdit: TEdit;
    PasswordEdit: TEdit;
    Label1: TLabel;
    Label2: TLabel;
    LoginBitBtn: TBitBtn;
    CancelBitBtn: TBitBtn;
    UserTable: TTable;
    SecurityTable: TTable;
    procedure FormCreate(Sender: TObject);
    procedure LoginBitBtnClick(Sender: TObject);
  private
    { Private declarations }
    CountDown: Integer;
  public
    { Public declarations }
  end;

var
  LoginDialogForm: TLoginDialogForm;

implementation

{$R *.DFM}

procedure TLoginDialogForm.FormCreate(Sender: TObject);
begin
  CountDown := 3;
end;

procedure TLoginDialogForm.LoginBitBtnClick(Sender: TObject);
begin

  {------------------------------------------------------------
  When ModalResult is mrNone the button will NOT close
  the dialog. Make sure the user is not allowed to close
  the dialog and slip away unless he really is known to
  and allowed into the system.
  -------------------------------------------------------------}

  ModalResult := mrNone;

  {-------------------------------------------------------------
  Check that user has provided all pieces...
  -------------------------------------------------------------}

  if UserNameEdit.Text = '' then
  begin
    MessageBeep( $FFFF ) ;
    Application.MessageBox( 'Missing user name.',
                            'Login Error',
                            MB_ICONEXCLAMATION + MB_OK );
    UserNameEdit.SetFocus ;
    exit; { nothing more to do here, short circuit out }
  end;

  if PasswordEdit.text = '' then
  begin
    MessageBeep( $FFFF ) ;
    Application.MessageBox( 'Missing password.',
                            'Login Error',
                            MB_ICONEXCLAMATION + MB_OK );
    PasswordEdit.SetFocus ;
    exit; { nothing more to do here, short circuit out }
  end;

  {-----------------------------------------------------------
  At this point the user has certainly provided both a user
  name and a password. Let's see if they are valid.
  -----------------------------------------------------------}

  try { Exception handling block. }

    try { Exception protection block. }

      {-------------------------------------------------------
      Programattically release the secret password and
      open the security table.
      -------------------------------------------------------}

      Session.AddPassword('YTIRUCES');
      SecurityTable.open;

      {-------------------------------------------------------
      See if the user name and password are in the table. If
      they are not, raise appropriate exception and fall
      through to the exception protection and handling blocks.
      -------------------------------------------------------}

      if not SecurityTable.FindKey([UserNameEdit.Text,
                                    PasswordEdit.Text])
      then
        raise ELoginInvalidInfo.create( 'Invalid login info.' );

      {-------------------------------------------------------
      The user is known to the system. See if he belongs to
      a user group. (In other words, assess the user's
      access privlege.)
      -------------------------------------------------------}

      if SecurityTable.FieldByName( 'USER_GROUP' ).isNull
      then
        raise ELoginMissingGroup.create( 'No access privlege.' );

      {-------------------------------------------------------
      Information provided so far is correct, and the user
      has access privlege to the system. Store the known
      information in the CurrentUser object.
      -------------------------------------------------------}

      CurrentUser.UserId :=
        SecurityTable.FieldByName( 'USER_ID' ).AsInteger ;

      CurrentUser.UserName :=
        SecurityTable.FieldByName( 'USER_NAME' ).AsString ;

      CurrentUser.UserGroup :=
        SecurityTable.FieldByName( 'USER_GROUP' ).AsInteger ;

      {-------------------------------------------------------
      Now retrieve any other pertinent information from the
      USER table.
      -------------------------------------------------------}

      UserTable.open ;

      {-------------------------------------------------------
      First find the user's record in the USER table. Note
      that we search for the CurrentUser.UserId AND we raise
      an appropriate exception if that key is not found in the
      user table. Actually this should NEVER happen, since
      there is a referential integrity constraint between the
      SECURITY table and the USER table. However, since this
      is programming for security, the code must account for
      the impossible.

      NOTE: With local database tables it is indeed possible
      for referential integrity constraits to be corrupted
      under certain circumstances.
      -------------------------------------------------------}

      if not UserTable.FindKey( [CurrentUser.UserId] ) then
        raise ELoginCorrupt.Create( 'Obsolete login.' );

      {-------------------------------------------------------
      All user information is available, and can be loaded
      into the CurrentUser object.
      -------------------------------------------------------}

      CurrentUser.FirstName :=
        UserTable.FieldByName( 'FIRST_NAME' ).AsString ;

      CurrentUser.LastName :=
        UserTable.FieldByName( 'LAST_NAME' ).AsString ;

      {-------------------------------------------------------
      The user is fully acknowledged and has appropriate
      access privileges. All user information is available
      and loaded in the CurrentUser object. Login was
      successful. Allow the dialog to close with an appropriate
      positive exit code.
      -------------------------------------------------------}

      ModalResult := mrOK;

    finally

      {-------------------------------------------------------
      If any exception was raised, you first have to close all
      open resources and above all REMOVE any given password.
      -------------------------------------------------------}

      UserTable.Close ;
      SecurityTable.Close ;
      Session.RemovePassword( 'YTIRUCES' );

    end;  { Exception protection block. }

  except

    {---------------------------------------------------------
    ...then the exception can be handled appropriately.
    ---------------------------------------------------------}

    on EDataBaseError do
    begin

      {-------------------------------------------------------
      Something went wrong with database access. Login has
      to be aborted. The dialog will be closed with a negative
      exit code.
      -------------------------------------------------------}

      MessageBeep( $FFFF );
      Application.MessageBox( 'Internal problems. Access denied.',
                              'Login Error',
                              MB_ICONSTOP + MB_OK);

      ModalResult := mrCancel ;

    end ;

    on E: ELoginException do
    begin

      {-------------------------------------------------------
      Something went wrong during user's login attempt. Show
      the exception message, and give the user another
      chance. If the user has no more chances available,
      close the dialog with a negative exit code.
      -------------------------------------------------------}

      MessageBeep( $FFFF );
      MessageDlg( E.Message, mtError, [mbOK], 0 ) ;

      dec( CountDown ) ;

      if CountDown <= 0 then ModalResult := mrCancel;

    end;

  end; { Exception handling block. }

end; { TLoginDialogForm.LoginBitBtnClick }

end.
